@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2012 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -13,7 +13,16 @@
 
 
 #include <stdarg.h>
-#include "api.h"
+#include "analyze-linkage.h"
+#include "api-structures.h"
+#include "post-process.h"
+#include "string-set.h"
+#include "structures.h"
+#include "word-utils.h"
+
+#ifdef USE_FAT_LINKAGES
+#include "and.h"
+#include "count.h"
 
 /**
  * The functions in this file do several things: () take a linkage
@@ -28,8 +37,8 @@
 typedef struct patch_element_struct Patch_element;
 struct patch_element_struct
 {
-	char used;   /* TRUE if this link is used, else FALSE  */
-	char changed;/* TRUE if this link changed, else FALSE  */
+	bool used;   /* TRUE if this link is used, else FALSE  */
+	bool changed;/* TRUE if this link changed, else FALSE  */
 	int newl;    /* the new value of the left end          */
 	int newr;    /* the new value of the right end         */
 };
@@ -53,7 +62,6 @@ struct analyze_context_s
 	Patch_element patch_array[MAX_LINKS];
 };
 
-#ifdef USE_FAT_LINKAGES
 typedef struct CON_node_struct CON_node;
 typedef struct CON_list_struct CON_list;
 typedef struct DIS_list_struct DIS_list;
@@ -142,10 +150,8 @@ static void free_sublinkage(Sublinkage *s)
 	for (i = 0; i < MAX_LINKS; i++) {
 		if (s->link[i] != NULL) exfree_link(s->link[i]);
 	}
-	xfree(s->link, MAX_LINKS*sizeof(Link));
-#ifdef USE_FAT_LINKAGES
+	xfree(s->link, MAX_LINKS*sizeof(Link *));
 	xfree(s, sizeof(Sublinkage));
-#endif /* USE_FAT_LINKAGES */
 }
 
 #ifdef USE_FAT_LINKAGES
@@ -452,7 +458,7 @@ static int advance_CON(CON_node *);
  * Cyclically advance the current state of this DIS node.
  * If it's now at the beginning of its cycle, return FALSE;
  * otherwise return TRUE. Together with the advance_CON()
- * function, this can be used to iterate over the entire 
+ * function, this can be used to iterate over the entire
  * DIS_CON tree.
  */
 static int advance_DIS(DIS_node * dn)
@@ -469,7 +475,7 @@ static int advance_DIS(DIS_node * dn)
  * Cyclically advance the current state of this CON node.
  * If it's now at the beginning of its cycle return FALSE,
  * otherwise return TRUE. Together with the advance_CON()
- * function, this can be used to iterate over the entire 
+ * function, this can be used to iterate over the entire
  * DIS_CON tree.
  */
 static int advance_CON(CON_node * cn)
@@ -747,13 +753,6 @@ static int link_cost(Parse_info pi)
 	return lcost;
 }
 
-static int null_cost(Parse_info pi)
-{
-  /* computes the number of null links in the linkage */
-  /* No one seems to care about this -- ALB */
-  return 0;
-}
-
 static int unused_word_cost(Parse_info pi)
 {
 	int lcost, i;
@@ -763,14 +762,14 @@ static int unused_word_cost(Parse_info pi)
 	return lcost;
 }
 
-/** 
+/**
  * Computes the cost of the current parse of the current sentence
  * due to the cost of the chosen disjuncts.
  */
-static float disjunct_cost(Parse_info pi)
+static double disjunct_cost(Parse_info pi)
 {
 	int i;
-	float lcost;
+	double lcost;
 	lcost =  0.0;
 	for (i = 0; i < pi->N_words; i++)
 	{
@@ -780,6 +779,65 @@ static float disjunct_cost(Parse_info pi)
 	return lcost;
 }
 
+/**
+ * This returns a string that is the the GCD of the two given strings.
+ * If the GCD is equal to one of them, a pointer to it is returned.
+ * Otherwise a new string for the GCD is xalloced and put on the
+ * "free later" list.
+ */
+const char * intersect_strings(Sentence sent, const char * s, const char * t)
+{
+	int i, j, d;
+	const char *w, *s0;
+	char u0[MAX_TOKEN_LENGTH]; /* Links are *always* less than 10 chars long */
+	char *u;
+	if (strcmp(s,t)==0) return s;  /* would work without this */
+	i = strlen(s);
+	j = strlen(t);
+	if (j > i) {
+		w = s; s = t; t = w;
+	}
+	/* s is now the longer (at least not the shorter) string */
+	u = u0;
+	d = 0;
+	s0 = s;
+	while (*t != '\0') {
+		if ((*s == *t) || (*t == '*')) {
+			*u = *s;
+		} else {
+			d++;
+			if (*s == '*') *u = *t;
+			else *u = '^';
+		}
+		s++; t++; u++;
+	}
+	if (d==0) {
+		return s0;
+	} else {
+		strcpy(u, s);   /* get the remainder of s */
+		return string_set_add(u0, sent->string_set);
+	}
+}
+
+/**
+ * The name of the link is set to be the GCD of the names of
+ * its two endpoints. Must be called after each extract_links(),
+ * etc. since that call issues a brand-new set of links into
+ * parse_info.
+ */
+static void compute_link_names(Sentence sent)
+{
+	int i;
+	Parse_info pi = sent->parse_info;
+
+	for (i = 0; i < pi->N_links; i++)
+	{
+		pi->link_array[i].name = intersect_strings(sent,
+			connector_get_string(pi->link_array[i].lc),
+			connector_get_string(pi->link_array[i].rc));
+	}
+}
+
 #ifdef USE_FAT_LINKAGES
 /**
  * Returns TRUE if string s represents a strictly smaller match set
@@ -814,28 +872,7 @@ static int strictly_smaller_name(const char * s, const char * t)
 	}
 	return (strictness > 0);
 }
-#endif /* USE_FAT_LINKAGES */
 
-/**
- * The name of the link is set to be the GCD of the names of
- * its two endpoints. Must be called after each extract_links(),
- * etc. since that call issues a brand-new set of links into 
- * parse_info.
- */
-static void compute_link_names(Sentence sent)
-{
-	int i;
-	Parse_info pi = sent->parse_info;
-
-	for (i = 0; i < pi->N_links; i++)
-	{
-		pi->link_array[i].name = intersect_strings(sent,
-			connector_get_string(pi->link_array[i].lc),
-			connector_get_string(pi->link_array[i].rc));
-	}
-}
-
-#ifdef USE_FAT_LINKAGES
 /**
  * This fills in the sublinkage->link[].name field.  We assume that
  * link_array[].name have already been filled in.  As above, in the
@@ -880,7 +917,6 @@ static void compute_pp_link_names(Sentence sent, Sublinkage *sublinkage)
 		}
 	}
 }
-#endif /* USE_FAT_LINKAGES */
 
 /********************** exported functions *****************************/
 
@@ -890,7 +926,7 @@ void init_analyze(Sentence s)
 
 	if (NULL == actx)
 	{
-		actx = (analyze_context_t *) malloc (sizeof(analyze_context_t));
+		actx = (analyze_context_t *) xalloc (sizeof(analyze_context_t));
 		s->analyze_ctxt = actx;
 	}
 
@@ -899,11 +935,10 @@ void init_analyze(Sentence s)
 
 void free_analyze(Sentence s)
 {
-	if (s->analyze_ctxt != NULL) free(s->analyze_ctxt);
+	if (s->analyze_ctxt != NULL) xfree(s->analyze_ctxt, sizeof(analyze_context_t));
 	s->analyze_ctxt = NULL;
 }
 
-#ifdef USE_FAT_LINKAGES
 /**
  * This uses link_array.  It enumerates and post-processes
  * all the linkages represented by this one.  We know this contains
@@ -942,9 +977,8 @@ Linkage_info analyze_fat_linkage(Sentence sent, Parse_Options opts, int analyze_
 	{
 		li.disjunct_cost = disjunct_cost(pi);
 	}
-	li.null_cost = null_cost(pi);
 	li.link_cost = link_cost(pi);
-	li.corpus_cost = -1.0f;
+	li.corpus_cost = -1.0;
 	li.and_cost = 0;
 	li.andlist = NULL;
 
@@ -1101,7 +1135,7 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 
 	/* The code below can be used to generate the "islands" array.
 	 * For this to work, however, you have to call "build_digraph"
-	 * first (as in analyze_fat_linkage). and then "free_digraph".
+	 * first (as in analyze_fat_linkage), and then "free_digraph".
 	 */
 	pp = post_process(postprocessor, opts, sent, sublinkage, TRUE);
 
@@ -1122,9 +1156,8 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	{
 		li.disjunct_cost = disjunct_cost(pi);
 	}
-	li.null_cost = null_cost(pi);
 	li.link_cost = link_cost(pi);
-	li.corpus_cost = -1.0f;
+	li.corpus_cost = -1.0;
 
 	if (pp == NULL)
 	{
@@ -1133,13 +1166,12 @@ Linkage_info analyze_thin_linkage(Sentence sent, Parse_Options opts, int analyze
 	else if (pp->violation != NULL)
 	{
 		li.N_violations++;
+		li.pp_violation_msg = pp->violation;
 	}
 
 	free_sublinkage(sublinkage);
 #ifdef USE_FAT_LINKAGES
 	free_digraph(actx, pi);
-#else
-	xfree(sublinkage, sizeof(Sublinkage));
 #endif /* USE_FAT_LINKAGES */
 	return li;
 }
@@ -1174,7 +1206,7 @@ void extract_thin_linkage(Sentence sent, Parse_Options opts, Linkage linkage)
 static void prt_lol(Sentence sent , List_o_links *lol)
 {
 	/* It appears that the list of links is always even in length:
-	 * The head word first, followed by a modifier. 
+	 * The head word first, followed by a modifier.
 	 */
 	while (lol)
 	{
@@ -1204,7 +1236,7 @@ static void prt_dis_list(Sentence sent, DIS_list *dis)
 			prt_lol(sent, dis->dn->lol);
 			printf("]");
 		}
-		else 
+		else
 		{
 			int wd = dis->dn->word;
 			printf("%s ", sent->word[wd].string);
