@@ -4,18 +4,23 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
 
-#include "api.h"
-
 #ifdef USE_FAT_LINKAGES
 
+#include "and.h"
+#include "api-structures.h"
+#include "dict-structures.h"
+#include "structures.h"
+#include "string-set.h"
+
+#include "count.h"
 #include "disjunct-utils.h"
+#include "word-utils.h"
 
 /*
                              Notes about AND
@@ -431,49 +436,7 @@ static int disjunct_types_equal(Disjunct * d1, Disjunct * d2)
 	if ((e1!=NULL) || (e2!=NULL)) return FALSE;
 	return TRUE;
 }
-#endif /* USE_FAT_LINKAGES */
 
-/**
- * This returns a string that is the the GCD of the two given strings.
- * If the GCD is equal to one of them, a pointer to it is returned.
- * Otherwise a new string for the GCD is xalloced and put on the
- * "free later" list.
- */
-const char * intersect_strings(Sentence sent, const char * s, const char * t)
-{
-	int i, j, d;
-	const char *w, *s0;
-	char u0[MAX_TOKEN_LENGTH]; /* Links are *always* less than 10 chars long */
-	char *u;
-	if (strcmp(s,t)==0) return s;  /* would work without this */
-	i = strlen(s);
-	j = strlen(t);
-	if (j > i) {
-		w = s; s = t; t = w;
-	}
-	/* s is now the longer (at least not the shorter) string */
-	u = u0;
-	d = 0;
-	s0 = s;
-	while (*t != '\0') {
-		if ((*s == *t) || (*t == '*')) {
-			*u = *s;
-		} else {
-			d++;
-			if (*s == '*') *u = *t;
-			else *u = '^';
-		}
-		s++; t++; u++;
-	}
-	if (d==0) {
-		return s0;
-	} else {
-		strcpy(u, s);   /* get the remainder of s */
-		return string_set_add(u0, sent->string_set);
-	}
-}
-
-#ifdef USE_FAT_LINKAGES
 /**
  * Two connectors are said to be equal if they are of the same type
  * (defined above), they have the same multi field, and they have
@@ -795,7 +758,7 @@ void print_AND_statistics(Sentence sent)
 	printf("Number of disjuncts in the table: %d\n", sent->and_data.STAT_N_disjuncts);
 	if (sent->and_data.LT_size != 0) {
 	  printf("average list length: %f\n",
-			 (float)sent->and_data.STAT_N_disjuncts/sent->and_data.LT_size);
+			 (double)sent->and_data.STAT_N_disjuncts/sent->and_data.LT_size);
 	}
 	printf("Number of equality tests: %d\n", sent->and_data.STAT_calls_to_equality_test);
 }
