@@ -4,8 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -20,21 +19,20 @@
 #ifndef _API_TYPES_H_
 #define _API_TYPES_H_
 
-#define MAX_TOKEN_LENGTH 250       /* maximum number of chars in a token */
+#define MAX_TOKEN_LENGTH 250     /* Maximum number of chars in a token */
 
 /* MAX_SENTENCE cannot be more than 254, because word MAX_SENTENCE+1 is 
  * used to indicate that nothing can connect to this connector, and this 
  * should fit in one byte (if the word field of a connector is an 
  * unsigned char).
  */
-#define MAX_SENTENCE 250      /* maximum number of words in a sentence */
+#define MAX_SENTENCE 250       /* Maximum number of words in a sentence */
 #define MAX_LINKS (2*MAX_SENTENCE-3) /* maximum number of links allowed */
 
-typedef char Boolean;
-
 /* Widely used private typedefs */
 #ifdef USE_FAT_LINKAGES
 typedef struct And_data_s And_data;
+typedef struct DIS_node_struct DIS_node;
 #endif /* USE_FAT_LINKAGES */
 typedef struct Connector_struct Connector;
 typedef struct Cost_Model_s Cost_Model;
@@ -62,11 +60,15 @@ typedef struct Link_s Link;
 typedef struct List_o_links_struct List_o_links;
 typedef struct Parse_set_struct Parse_set;
 typedef struct String_set_s String_set;
+typedef struct Tokenizer_struct Tokenizer;
 typedef struct Word_struct Word;
 typedef struct X_table_connector_struct X_table_connector;
 
 
+/* Post-processing structures */
 typedef struct pp_knowledge_s pp_knowledge;
+typedef struct pp_linkset_s pp_linkset;
+typedef struct PP_node_struct PP_node;
 
 typedef struct corpus_s Corpus;
 typedef struct sense_s Sense;
