@@ -4,8 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -13,11 +12,14 @@
 
 #include <stdarg.h>
 #include <string.h>
-#include "api.h"
+
+#include "api-structures.h"
 #include "error.h"
+#include "externs.h"
 #include "constituents.h"
 #include "post-process.h"
 #include "print-util.h"
+#include "string-set.h"
 
 #define MAXCONSTITUENTS 8192
 #define OPEN_BRACKET '['
@@ -1445,7 +1447,9 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 	return numcon_subl;
 }
 
-static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage, int numcon_total)
+static char * 
+exprint_constituent_structure(con_context_t *ctxt,
+                              Linkage linkage, int numcon_total)
 {
 	int have_opened = 1;
 	int c, w;
@@ -1453,7 +1457,7 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 	int rightdone[MAXCONSTITUENTS];
 	int best, bestright, bestleft;
 	Sentence sent;
-	char s[100], * p;
+	char s[MAX_WORD], *p;
 	String * cs = string_new();
 
 	assert (numcon_total < MAXCONSTITUENTS, "Too many constituents (b)");
@@ -1468,12 +1472,11 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 	if (verbosity >= 2)
 		printf("\n");			
 
+	/* Skip left wall; don't skip right wall, since it may
+	 * have constituent boundaries. */
 	for (w = 1; w < linkage->num_words; w++)
 	{
-		/* Skip left wall; don't skip right wall, since it may
-		   have constituent boundaries */
-
-		while(1)
+		while (1)
 		{
 			best = -1;
 			bestright = -1;
@@ -1481,7 +1484,8 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 			{
 				if ((ctxt->constituent[c].left == w) &&
 					(leftdone[c] == 0) && (ctxt->constituent[c].valid == 1) &&
-					(ctxt->constituent[c].right >= bestright)) {
+					(ctxt->constituent[c].right >= bestright))
+				{
 					best = c;
 					bestright = ctxt->constituent[c].right;
 				}
@@ -1501,24 +1505,25 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 		if (have_opened && (w < linkage->num_words - 1))
 		{
 			char *p;
-			/* XXX FIXME -- using alternatives[0] is wrong here,
-			 * especially for Russian. We'll punt for now, though.
-			 * Consider using linkage->word[w] instead. */
-			strcpy(s, sent->word[w].alternatives[0]);
+			strncpy(s, linkage->word[w], MAX_WORD);
+			s[MAX_WORD-1] = 0;
 
 			/* Constituent processing will crash if the sentence contains
 			 * square brackets, so we have to do something ... replace
-			 * them with curly braces ... will have to do.
+			 * them with curly braces ... this is a terrible hack, but
+			 * will have to do; for now.  A better solution would be to
+			 * allow the user to specify some reserved char as the
+			 * bracket symbol, e.g. SOH and EOT or something like that.
 			 */
 			p = strchr(s, OPEN_BRACKET);
-			while(p)
+			while (p)
 			{
 				*p = '{';
 				p = strchr(p, OPEN_BRACKET);
 			}
 
 			p = strchr(s, CLOSE_BRACKET);
-			while(p)
+			while (p)
 			{
 				*p = '}';
 				p = strchr(p, CLOSE_BRACKET);
@@ -1531,7 +1536,7 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 			append_string(cs, "%s ", s);
 		}
 
-		while(1)
+		while (1)
 		{
 			best = -1;
 			bestleft = -1;
@@ -1650,10 +1655,10 @@ static char * print_flat_constituents(Linkage linkage)
 	 * This was discovered only after much work. Bummer.
 	 */
 	char * p;
-	con_context_t *ctxt = (con_context_t *) malloc (sizeof(con_context_t));
+	con_context_t *ctxt = (con_context_t *) xalloc (sizeof(con_context_t));
 	memset(ctxt, 0, sizeof(con_context_t));
 	p = do_print_flat_constituents(ctxt, linkage);
-	free(ctxt);
+	xfree(ctxt, sizeof(con_context_t));
 	return p;
 }
 
@@ -1817,6 +1822,10 @@ CNode * linkage_constituent_tree(Linkage linkage)
 	return root;
 }
 
+/* Make the compiler shut up about the deprecated functions */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
 void linkage_free_constituent_tree(CNode * n)
 {
 	CNode *m, *x;
