@@ -4,8 +4,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -19,8 +18,12 @@
  pp_lexer.h
 ***********************************************************************/
 
-#include "api.h"
 #include "error.h"
+#include "externs.h"
+#include "pp_knowledge.h"
+#include "pp_lexer.h"
+#include "pp_linkset.h"
+#include "string-set.h"
 #include "utilities.h"
 
 #define PP_MAX_UNIQUE_LINK_NAMES 1024  /* just needs to be approximate */
@@ -100,7 +103,7 @@ static pp_linkset *read_link_set(pp_knowledge *k,
   int n_strings,i;
   pp_linkset *ls;
   if (!pp_lexer_set_label(k->lt, label)) {
-    if (verbosity>0)
+    if (1 < verbosity)
       printf("PP warning: Link set %s not defined: assuming empty.\n",label);
     n_strings = 0;
   }
@@ -148,7 +151,7 @@ static void read_connected_rule(pp_knowledge *k, const char *label)
   if (!pp_lexer_set_label(k->lt, label))
     {
       k->connected_rules[0].msg=0;  /* rule not there */
-      if (verbosity>0) printf("PP warning: Not using 'link is connected' rule\n");
+      if (1 < verbosity) printf("PP warning: Not using 'link is connected' rule\n");
       return;
     }
   if (pp_lexer_count_tokens_of_label(k->lt)>1)
@@ -168,8 +171,8 @@ static void read_form_a_cycle_rules(pp_knowledge *k, const char *label)
   const char **tokens;
   if (!pp_lexer_set_label(k->lt, label)) {
       k->n_form_a_cycle_rules = 0;
-      if (verbosity>0)
-	printf("PP warning: Not using any 'form a cycle' rules\n");
+      if (1 < verbosity)
+          printf("PP warning: Not using any 'form a cycle' rules\n");
   }
   else {
     n_commas = pp_lexer_count_commas_of_label(k->lt);
@@ -211,7 +214,7 @@ static void read_bounded_rules(pp_knowledge *k, const char *label)
   int n_commas, n_tokens, r;
   if (!pp_lexer_set_label(k->lt, label)) {
       k->n_bounded_rules = 0;
-      if (verbosity>0) printf("PP warning: Not using any 'bounded' rules\n");
+      if (1 < verbosity) printf("PP warning: Not using any 'bounded' rules\n");
   }
   else {
     n_commas = pp_lexer_count_commas_of_label(k->lt);
@@ -253,7 +256,7 @@ static void read_contains_rules(pp_knowledge *k, const char *label,
   const char **tokens;
   if (!pp_lexer_set_label(k->lt, label)) {
       *nRules = 0;
-      if (verbosity>0) printf("PP warning: Not using any %s rules\n", label);
+      if (1 < verbosity) printf("PP warning: Not using any %s rules\n", label);
   }
   else {
     n_commas = pp_lexer_count_commas_of_label(k->lt);
