@@ -4,18 +4,29 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
 
-#include "api.h"
+#include "api-structures.h"
+#include "build-disjuncts.h"
+#include "count.h"
 #include "disjunct-utils.h"
+#include "externs.h"
+#include "preparation.h"
+#include "print.h"
+#include "prune.h"
 #include "resources.h"
+#include "structures.h"
+#include "word-utils.h"
 
 #ifdef USE_FAT_LINKAGES
+
+#include "and.h"
+#include "massage.h"
+
 void free_deletable(Sentence sent)
 {
 	int w;
