@@ -1,12 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright 2013 Linas Vepstas                                          */
+/* Copyright 2013, 2014 Linas Vepstas                                    */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */ 
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -16,60 +15,71 @@
 #include <stdarg.h>
 
 #include "print.h"
+#include "externs.h"
 #include "api-structures.h"
 #include "corpus/corpus.h"
 #include "idiom.h"
 #include "print-util.h"
 #include "string-set.h"
+#include "structures.h" /* needed for EMPTY_WORD */
 #include "utilities.h"
 #include "word-utils.h"
 
 #define LEFT_WALL_DISPLAY  ("LEFT-WALL")   /* the string to use to show the wall */
 #define LEFT_WALL_SUPPRESS ("Wd") /* If this connector is used on the wall, */
                                   /* then suppress the display of the wall. */
-#define RIGHT_WALL_DISPLAY  ("RIGHT-WALL")   /* the string to use to show the wall */
+#define RIGHT_WALL_DISPLAY  ("RIGHT-WALL") /* the string to use to show the wall */
 #define RIGHT_WALL_SUPPRESS ("RW") /* If this connector is used on the wall, */
 
 /* The Russian dictionary makes use of the empty word to deal with
  * the splitting of words into variable-length word-counts */
-#define EMPTY_WORD ("=.zzz") /* pure whitespace */
 #define EMPTY_WORD_SUPPRESS ("ZZZ") /* link to pure whitespace */
 
-#define SUFFIX_WORD ("=")  /* suffixes start with this */
-#define SUFFIX_WORD_L 1     /* length of above */
+#define SUFFIX_WORD ("=")      /* suffixes start with this */
+#define SUFFIX_WORD_L 1        /* length of above */
 #define SUFFIX_SUPPRESS ("LL") /* suffix links start with this */
-#define SUFFIX_SUPPRESS_L 2  /* length of above */
+#define SUFFIX_SUPPRESS_L 2    /* length of above */
 
-#define HIDE_SUFFIX   (!display_suffixes)
+#define HIDE_MORPHO   (!display_morphology)
 
-/* XXX FIXME
- * There are several problems with the handling of suffixes here.
- * 1) Valid English constructions like "I think that 2 + 2 = 4" display
- *    bizarrely.  This needs fixing.
- * 2) Engish sentences like "this is a hey= =.zzz test" display
- *    unexpectely. (because the suffixes were contracted.
- * In breif, the mechanism should be disabled for English.
+/**
+ * Return TRUE if the word is a suffix.
+ *
+ * Suffixes have the form =asdf.asdf and "null" suffixes have the form =.asdf.
+ * Ordinary equals signs appearing in regular text are either = or =[!].
  */
+static Boolean is_suffix(const char* w)
+{
+	if (0 != strncmp(SUFFIX_WORD, w, SUFFIX_WORD_L)) return FALSE;
+	if (1 == strlen(w)) return FALSE;
+#if SUBSCRIPT_MARK == '.'
+	if (0 == strcmp("=[!]", w)) return FALSE;
+	if (0 == strcmp("=.v", w)) return FALSE;
+	if (0 == strcmp("=.eq", w)) return FALSE;
+#endif
+	return TRUE;
+}
 
-static void set_centers(const Linkage linkage, int center[], Boolean print_word_0, int N_words_to_print)
+static void
+set_centers(const Linkage linkage, int center[],
+            Boolean print_word_0, int N_words_to_print)
 {
 	int i, len, tot;
-	Boolean display_suffixes = linkage->opts->display_suffixes;
+	Boolean display_morphology = linkage->opts->display_morphology;
 
 	tot = 0;
 	if (print_word_0) i=0; else i=1;
 	for (; i < N_words_to_print; i++)
 	{
-		/* Ignore suffixes */
-		if (HIDE_SUFFIX &&
-		    0 == strncmp (SUFFIX_WORD, linkage->word[i], SUFFIX_WORD_L))
+		/* Ignore morphological analysis. */
+		if (HIDE_MORPHO && is_suffix(linkage->word[i]))
 		{
 			center[i] = tot;
 			tot++;  // hack alert -- a trailing blank gets printed anyway ...
 			continue;
 		}
 
-		/* Centers obtained by counting the characters, 
+		/* Centers obtained by counting the characters,
 		 * not the bytes in the string.
 		 * len = strlen(linkage->word[i]);
 		 * len = mbsrtowcs(NULL, &linkage->word[i], 0, &mbss);
@@ -120,7 +130,7 @@ static void print_a_link(String * s, const Linkage linkage, int link)
 	Dictionary dict = sent->dict;
 	int l, r;
 	const char *label, *llabel, *rlabel;
-	
+
 	l      = linkage_get_link_lword(linkage, link);
 	r      = linkage_get_link_rword(linkage, link);
 	label  = linkage_get_link_label(linkage, link);
@@ -130,14 +140,14 @@ static void print_a_link(String * s, const Linkage linkage, int link)
 	if ((l == 0) && dict->left_wall_defined) {
 		left_append_string(s, LEFT_WALL_DISPLAY,"               ");
 	} else if ((l == (linkage_get_num_words(linkage)-1)) && dict->right_wall_defined) {
-		left_append_string(s, RIGHT_WALL_DISPLAY,"               ");	
+		left_append_string(s, RIGHT_WALL_DISPLAY,"               ");
 	} else {
 		left_append_string(s, linkage_get_word(linkage, l), "               ");
 	}
 	left_append_string(s, llabel, "     ");
-	append_string(s, "   <---"); 
+	append_string(s, "   <---");
 	left_append_string(s, label, "-----");
-	append_string(s, "->  "); 
+	append_string(s, "->  ");
 	left_append_string(s, rlabel, "     ");
 	append_string(s, "     %s\n", linkage_get_word(linkage, r));
 }
@@ -147,7 +157,7 @@ static void print_a_link(String * s, const Linkage linkage, int link)
  * To the left of each link, print the sequence of domains it is in.
  * Printing a domain means printing its type.
  * Takes info from pp_link_array and pp and chosen_words.
- */    
+ */
 char * linkage_print_links_and_domains(const Linkage linkage)
 {
 	int link, longest, j;
@@ -159,7 +169,7 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 	longest = 0;
 	for (link=0; link<N_links; link++) {
 		if (linkage_get_link_lword(linkage, link) == -1) continue;
-		if (linkage_get_link_num_domains(linkage, link) > longest) 
+		if (linkage_get_link_num_domains(linkage, link) > longest)
 			longest = linkage_get_link_num_domains(linkage, link);
 	}
 	for (link=0; link<N_links; link++) {
@@ -179,10 +189,10 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 		append_string(s, "P.P. violations:\n");
 		append_string(s, "        %s\n\n", linkage_get_violation_name(linkage));
 	}
-	
+
 	links_string = string_copy(s);
 	string_delete(s);
-	return links_string; 
+	return links_string;
 }
 
 char * linkage_print_senses(Linkage linkage)
@@ -226,35 +236,39 @@ char * linkage_print_disjuncts(const Linkage linkage)
 #ifdef USE_CORPUS
 	double score;
 #endif
-	double cost;
-	const char * infword;
 	const char * dj;
-	char * djs;
+	char * djs, *mark;
 	int w;
 	String * s = string_new();
 	Sentence sent = linkage->sent;
 	int nwords = sent->length;
 
-	/* Decrement nwords, so as to ignore the RIGHT-WALL */
-	nwords --;
-
-	/* Loop over each word in the sentence (skipping LEFT-WALL, which is
-	 * word 0. */
-	for (w=1; w<nwords; w++)
+	/* Loop over each word in the sentence */
+	for (w = 0; w < nwords; w++)
 	{
+		int pad = 21;
+		double cost;
+		char infword[MAX_WORD];
 		Disjunct *disj = linkage->sent->parse_info->chosen_disjuncts[w];
 		if (NULL == disj) continue;
 
-		infword = disj->string;
+		/* Cleanup the subscript mark before printing. */
+		strncpy(infword, disj->string, MAX_WORD);
+		mark = strchr(infword, SUBSCRIPT_MARK);
+		if (mark) *mark = '.';
+
+		/* Make sure the glyphs align during printing. */
+		pad += strlen(infword) - utf8_strlen(infword);
 
 		dj = linkage_get_disjunct_str(linkage, w);
+		if (NULL == dj) dj = "";
 		cost = linkage_get_disjunct_cost(linkage, w);
 
 #ifdef USE_CORPUS
 		score = linkage_get_disjunct_corpus_score(linkage, w);
-		append_string(s, "%21s    %5.1f %6.3f %s\n", infword, cost, score, dj);
+		append_string(s, "%*s    %5.1f %6.3f %s\n", pad, infword, cost, score, dj);
 #else
-		append_string(s, "%21s    %5.1f  %s\n", infword, cost, dj);
+		append_string(s, "%*s    %5.1f  %s\n", pad, infword, cost, dj);
 #endif
 	}
 	djs = string_copy(s);
@@ -264,7 +278,7 @@ char * linkage_print_disjuncts(const Linkage linkage)
 
 /**
  * XXX TODO: port over the suppression of ZZZ links, and also
- * the HIDE_SUFFIX stuff, fom the ascii printing code, way below.
+ * the HIDE_MORPHO stuff, fom the ascii printing code, way below.
  */
 static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *pctx)
 {
@@ -289,7 +303,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 	N_wall_connectors = 0;
 	if (dict->left_wall_defined) {
 		suppressor_used = FALSE;
-		if (!opts->display_walls) 
+		if (!opts->display_walls)
 			for (j=0; j<N_links; j++) {
 				if (ppla[j]->l == 0) {
 					if (ppla[j]->r == linkage->num_words-1) continue;
@@ -299,7 +313,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 					}
 				}
 			}
-		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0)) 
+		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0))
 						|| (N_wall_connectors > 1) || opts->display_walls);
 	} else {
 		print_word_0 = TRUE;
@@ -316,9 +330,9 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 				}
 			}
 		}
-		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0)) 
+		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0))
 						|| (N_wall_connectors > 1) || opts->display_walls);
-	} 
+	}
 	else {
 		print_word_N = TRUE;
 	}
@@ -347,7 +361,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 		if ((j%7 == 0) && (j>0)) append_string(string,"\n");
 		j++;
 		append_string(string,"[%d %d %d",
-				ppla[link]->l-d, ppla[link]->r-d, 
+				ppla[link]->l-d, ppla[link]->r-d,
 				pctx->link_heights[link]);
 #ifdef USE_FAT_LINKAGES
 		if (ppla[link]->lc->label < 0) {
@@ -372,7 +386,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 
 	ps_string = string_copy(string);
 	string_delete(string);
-	return ps_string; 
+	return ps_string;
 }
 
 /**
@@ -386,7 +400,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 	Parse_info pi = sent->parse_info;
 	const char * chosen_words[MAX_SENTENCE];
 	Parse_Options opts = linkage->opts;
-	Boolean display_suffixes = opts->display_suffixes;
+	Boolean display_morphology = opts->display_morphology;
 
 	for (i=0; i<sent->length; i++)
 	{
@@ -410,37 +424,59 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 				xfree(s, l+1);
 			}
 			else
-				t = string_set_add("", sent->string_set);
+			{
+				/* Altrnative token island.
+				 * Show the internal word number and its list of alternatives. */
+				String * s = string_new();
+				const char ** a;
+				char * a_list;
+
+				append_string(s, "[%d", i);
+				for (a = sent->word[i].alternatives; *a; a++) {
+					append_string(s, " %s", *a);
+				}
+				append_string(s, "]");
+				a_list = string_copy(s);
+				t = string_set_add(a_list, sent->string_set);
+				string_delete(s);
+				exfree(a_list, strlen(a_list)+1);
+			}
 		}
 		else if (opts->display_word_subscripts)
 		{
-			t = pi->chosen_disjuncts[i]->string;        
+			t = pi->chosen_disjuncts[i]->string;
 			/* get rid of those ugly ".Ixx" */
 			if (is_idiom_word(t)) {
 				s = strdup(t);
-				u = strrchr(s, '.');
+				u = strrchr(s, SUBSCRIPT_MARK);
 				*u = '\0';
 				t = string_set_add(s, sent->string_set);
 				free(s);
+			} else {
+				/* Convert the badly-printing ^C into a period */
+				s = strdup(t);
+				u = strrchr(s, SUBSCRIPT_MARK);
+				if (u) *u = SUBSCRIPT_DOT;
+				t = string_set_add(s, sent->string_set);
+				free(s);
 			}
 
 			/* Suppress the empty word. */
-			if (0 == strcmp(t, EMPTY_WORD))
+			if (0 == strcmp(t, EMPTY_WORD_DOT))
 			{
 				t = string_set_add("", sent->string_set);
 			}
 
-			/* Concatenate the stem and the sufix together into one word */
-			if (HIDE_SUFFIX)
+			/* Concatenate the stem and the suffix together into one word */
+			if (HIDE_MORPHO)
 			{
-				if (0 == strncmp(t, SUFFIX_WORD, SUFFIX_WORD_L) &&
-				   pi->chosen_disjuncts[i-1])
+				if (is_suffix(t) && pi->chosen_disjuncts[i-1])
 				{
 					const char * stem = pi->chosen_disjuncts[i-1]->string;
 					size_t len = strlen(stem) + strlen (t);
 					char * join = (char *)malloc(len+1);
 					strcpy(join, stem);
-					u = strrchr(join, '.');
+					u = strrchr(join, SUBSCRIPT_MARK);
 
 					/* u can be null, if the the sentence happens to have
 					 * an equals sign in it, for other reasons. */
@@ -458,8 +494,7 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 				    (pi->chosen_disjuncts[i+1]))
 				{
 					const char * next = pi->chosen_disjuncts[i+1]->string;
-					if (0 == strncmp(next, SUFFIX_WORD, SUFFIX_WORD_L) &&
-					    0 != strcmp(next, EMPTY_WORD))
+					if (is_suffix(next) && 0 != strcmp(next, EMPTY_WORD_MARK))
 					{
 						t = string_set_add("", sent->string_set);
 					}
@@ -469,10 +504,10 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 		else
 		{
 			/* XXX This is wrong, since it fails to indicate what
-			 * was actally used for the parse, which might not actually 
+			 * was actally used for the parse, which might not actually
 			 * be alternative 0.  We should do like the above, and then
 			 * manually strip the subscript.
-			 * Except that this code is never ever reached, because 
+			 * Except that this code is never ever reached, because
 			 * opts->display_word_subscripts is always true...
 			 */
 			t = sent->word[i].alternatives[0];
@@ -496,13 +531,13 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 
 #define MAX_HEIGHT 30
 
-/** 
+/**
  * Print the indicated linkage into a utf8-diagram.
  * Works fine for general utf8 multi-byte sentences.
  * Links and connectors are still mostly assumed to be ASCII, though;
  * to fix this, grep for "strlen" in the code below, replace by utf8 len.
  *
- * Returned string is allocated with exalloc.  
+ * Returned string is allocated with exalloc.
  * Needs to be freed with linkage_free_diagram()
  */
 static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
@@ -516,9 +551,9 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	char connector[MAX_TOKEN_LENGTH];
 	unsigned int line_len, link_length;
 #ifdef USE_FAT_LINKAGES
-	Sublinkage *sublinkage=&(linkage->sublinkage[linkage->current]);
+	Sublinkage *sublinkage = &(linkage->sublinkage[linkage->current]);
 #else
-	Sublinkage *sublinkage=&(linkage->sublinkage);
+	Sublinkage *sublinkage = &(linkage->sublinkage);
 #endif /* USE_FAT_LINKAGES */
 	unsigned int N_links = sublinkage->num_links;
 	Link **ppla = sublinkage->link;
@@ -533,7 +568,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	char xpicture[MAX_HEIGHT][MAX_LINE];
 	size_t start[MAX_HEIGHT];
 
-	Boolean display_suffixes = opts->display_suffixes;
+	Boolean display_morphology = opts->display_morphology;
 
 	string = string_new();
 
@@ -542,11 +577,11 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	if (dict->left_wall_defined)
 	{
 		suppressor_used = FALSE;
-		if (!opts->display_walls) 
+		if (!opts->display_walls)
 		{
 			for (j=0; j<N_links; j++)
 			{
-				if ((ppla[j]->l == 0))
+				if (0 == ppla[j]->l)
 				{
 					if (ppla[j]->r == linkage->num_words-1) continue;
 					N_wall_connectors ++;
@@ -557,9 +592,9 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 				}
 			}
 		}
-		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0)) 
+		print_word_0 = (((!suppressor_used) && (N_wall_connectors != 0))
 						|| (N_wall_connectors > 1) || opts->display_walls);
-	} 
+	}
 	else
 	{
 		print_word_0 = TRUE;
@@ -581,9 +616,9 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 				}
 			}
 		}
-		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0)) 
+		print_word_N = (((!suppressor_used) && (N_wall_connectors != 0))
 						|| (N_wall_connectors > 1) || opts->display_walls);
-	} 
+	}
 	else
 	{
 		print_word_N = TRUE;
@@ -594,13 +629,13 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 
 	set_centers(linkage, center, print_word_0, N_words_to_print);
 	line_len = center[N_words_to_print-1]+1;
-	
+
 	for (k=0; k<MAX_HEIGHT; k++) {
 		for (j=0; j<line_len; j++) picture[k][j] = ' ';
 		picture[k][line_len] = '\0';
 	}
 	top_row = 0;
-	
+
 	for (link_length = 1; link_length < N_words_to_print; link_length++) {
 		for (j=0; j<N_links; j++) {
 			if (ppla[j]->l == -1) continue;
@@ -608,11 +643,11 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			  continue;
 			if (!print_word_0 && (ppla[j]->l == 0)) continue;
 			/* gets rid of the irrelevant link to the left wall */
-			if (!print_word_N && (ppla[j]->r == linkage->num_words-1)) continue;            
+			if (!print_word_N && (ppla[j]->r == linkage->num_words-1)) continue;
 			/* Get rid of links to empty words */
 			if (0 == strcmp(ppla[j]->name, EMPTY_WORD_SUPPRESS)) continue;
 
-			if (HIDE_SUFFIX &&
+			if (HIDE_MORPHO &&
 			    0 == strncmp(ppla[j]->name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L))
 				continue;
 
@@ -631,22 +666,22 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			/* we know it fits, so put it in this row */
 
 			pctx->link_heights[j] = row;
-			
+
 			if (2*row+2 > MAX_HEIGHT-1) {
 				append_string(string, "The diagram is too high.\n");
 				gr_string = string_copy(string);
 				string_delete(string);
-				return gr_string; 
+				return gr_string;
 			}
 			if (row > top_row) top_row = row;
-			
+
 			picture[row][cl] = '+';
 			picture[row][cr] = '+';
 			for (k=cl+1; k<cr; k++) {
 				picture[row][k] = '-';
 			}
 			s = ppla[j]->name;
-			
+
 			/* display_link_subscripts is always true ... hard-coded */
 			if (opts->display_link_subscripts)
 			{
@@ -667,17 +702,17 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			if (opts->display_link_subscripts)
 				k = strlen(connector);
 			else
-				for (t=connector; isupper((int)*t); t++) k++; /* uppercase len of conn*/
+				for (t = connector; isupper((int)*t); t++) k++; /* uppercase len of conn*/
 			if ((cl+cr-k)/2 + 1 <= cl) {
 				t = picture[row] + cl + 1;
 			} else {
-				t = picture[row] + (cl+cr-k)/2 + 1;
+				t = picture[row] + (cl + cr + 2 - k)/2;
 			}
 			s = connector;
 			if (opts->display_link_subscripts)
-				while((*s != '\0') && (*t == '-')) *t++ = *s++; 
+				while ((*s != '\0') && (*t == '-')) *t++ = *s++;
 			else
-				while(isupper((int)*s) && (*t == '-')) *t++ = *s++; 
+				while (isupper((int)*s) && (*t == '-')) *t++ = *s++;
 
 			/* now put in the | below this one, where needed */
 			for (k=0; k<row; k++) {
@@ -690,9 +725,9 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			}
 		}
 	}
-	
+
 	/* we have the link picture, now put in the words and extra "|"s */
-	
+
 	t = xpicture[0];
 	if (print_word_0) k = 0; else k = 1;
 	for (; k<N_words_to_print; k++) {
@@ -705,7 +740,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		*t++ = ' ';
 	}
 	*t = '\0';
-	
+
 	if (opts->display_short) {
 		for (k=0; picture[0][k] != '\0'; k++) {
 			if ((picture[0][k] == '+') || (picture[0][k] == '|')) {
@@ -733,13 +768,13 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		}
 		top_row = 2*top_row + 2;
 	}
-	
-	/* we've built the picture, now print it out */
-	
+
+	/* We've built the picture, now print it out. */
+
 	if (print_word_0) i = 0; else i = 1;
 
-	/* start locations, for each row.  These may vary, due to different
-	 * utf8 character widths */
+	/* Start locations, for each row.  These may vary, due to different
+	 * utf8 character widths. */
 	top_row_p1 = top_row + 1;
 	for (row = 0; row < top_row_p1; row++)
 		start[row] = 0;
@@ -787,7 +822,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 				for (j = k; (mbcnt < uwidth) && (xpicture[row][j] != '\0'); )
 				{
 					/* Copy exactly one multi-byte character to buf */
-					j += append_utf8_char(string, &xpicture[row][j]); 
+					j += append_utf8_char(string, &xpicture[row][j]);
 					mbcnt ++;
 				}
 				append_string(string, "\n");
@@ -797,10 +832,10 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	}
 	gr_string = string_copy(string);
 	string_delete(string);
-	return gr_string; 
+	return gr_string;
 }
 
-/** 
+/**
  * Print the indicated linkage as utf8-art intp the given string.
  * The width of the diagram is given by the terminal width, taken
  * from the parse options.
@@ -851,17 +886,29 @@ char * linkage_print_postscript(Linkage linkage, int mode)
 
 	ps = build_linkage_postscript_string(linkage, &ctx);
 	size = strlen(header(mode)) + strlen(ps) + strlen(trailer(mode)) + 1;
-	
+
 	qs = (char *) exalloc(sizeof(char)*size);
 	sprintf(qs, "%s%s%s", header(mode), ps, trailer(mode));
 	exfree(ps, strlen(ps)+1);
-	 
+
 	return qs;
 }
 
 void linkage_free_postscript(char * s)
 {
-		exfree(s, strlen(s)+1);
+	exfree(s, strlen(s)+1);
+}
+
+char * linkage_print_pp_msgs(Linkage linkage)
+{
+	if (linkage && linkage->info && linkage->info->pp_violation_msg)
+		return strdup(linkage->info->pp_violation_msg);
+	return strdup("");
+}
+
+void linkage_free_pp_msgs(char * s)
+{
+	exfree(s, strlen(s)+1);
 }
 
 void print_disjunct_counts(Sentence sent)
@@ -1266,3 +1313,173 @@ static const char * header(int mode)
 	if (mode==1) return header_string;
 	else return "";
 }
+
+/**
+ *   print_sentence_word_alternatives(sent, FALSE, display_func, NULL)
+ * Iterate over the sentence words and their alternatives.
+ * Handle each alternative using the display() function if it is supplied,
+ * or else just print them.
+ * The returned value is not relevant in ths case.
+ *
+ *   print_sentence_word_alternatives(sent, FALSE, NULL, token)
+ * Return the word and alternative index of first occurrence in the sentence
+ * of the given token. This is used to prevent duplicate inforamtion display
+ * for repeated morphemes (if there are multiples splits, each of several
+ * morphemes, then many of them may repeat).
+ *
+ *   print_sentence_word_alternatives(sent, TRUE, NULL, NULL)
+ * If debugprint is TRUE, this is a debug printout of the sentence.
+ * (The debug printouts are with level 0 because this function is
+ * invoked for debug on certain positive level.)
+ * The returned value is not relevant in ths case.
+ *
+ */
+
+struct tokenpos print_sentence_word_alternatives(Sentence sent,
+		Boolean debugprint, void (*display)(Dictionary, const char *),
+		const char * findtoken)
+{
+	int wi;   /* internal sentence word index */
+	int ai;   /* index of a word alternative */
+	int sentlen = sent->length;	     /* shortened if there is a right-wall */
+	int first_sentence_word = 0;       /* used for skipping a left-wall */
+	int word_split = FALSE;            /* !!word got split */
+	struct tokenpos tokenpos = {0, 0}; /* token index to prevent duplicates */
+
+	if (0 == sentlen)
+	{
+		/* It should not happen, but if it actually happens due to some
+		 * strange conditions, it's better not to abort the program. */
+		prt_error("Error: Sentence length is 0 (reason unknown)\n");
+		return tokenpos;
+	}
+
+	if (debugprint) lgdebug(1, "Words and alternatives:\n");
+	else if (findtoken)
+		; /* do nothing */
+	else
+	{
+		/* For analyzing words we need to ignore the left/right walls */
+		if ((sentlen >= 0) &&
+		 (0 == strcmp(sent->word[0].unsplit_word, LEFT_WALL_WORD)))
+			first_sentence_word = 1;
+		if ((sentlen >= 0) && (NULL != sent->word[sentlen-1].unsplit_word) &&
+		 (0 == strcmp(sent->word[sentlen-1].unsplit_word, RIGHT_WALL_WORD)))
+			sentlen--;
+
+		/* Find if a word got split. This is indicated by:
+		 * 1. More than one word in the sentence
+		 * (no check if it actually results from !!w1 w2 ...).
+		 * 2. A word containing more than one alternative. */
+		if (sentlen - first_sentence_word > 1)
+		{
+			word_split = TRUE;
+		}
+		else
+		{
+			for (wi=first_sentence_word; wi<sentlen; wi++)
+			{
+				Word w = sent->word[wi];
+
+				/* There should always be at least one alternative */
+				assert((NULL != w.alternatives) && (NULL != w.alternatives[0]) &&
+				 ('\0' != w.alternatives[0][0]), "Missing alt for word %d\n", wi);
+
+				if (NULL != w.alternatives[1])
+				{
+					word_split = TRUE;
+					break;
+				}
+			}
+		}
+		/* "String", because it can be a word, morpheme, or (TBD) idiom */
+		if (word_split && (NULL == display)) printf("String splits to:\n");
+	}
+
+	/* Iterate over sentence input words */
+	for (wi=first_sentence_word; wi<sentlen; wi++)
+	{
+		Word w = sent->word[wi];
+		int w_start = wi;   /* input word index */
+
+		if (debugprint) lgdebug(0, "  word%d: %s\n  ", wi, w.unsplit_word);
+
+		/* There should always be at least one alternative */
+		assert((NULL != w.alternatives) && (NULL != w.alternatives[0]) &&
+		 ('\0' != w.alternatives[0][0]), "Missing alt for word %d\n", wi);
+
+		/* Iterate over alternatives */
+		for (ai=0; ;  ai++)
+		{
+			Boolean alt_exists = w.alternatives[ai] != NULL;
+
+			if (alt_exists && debugprint)
+				lgdebug(0, "   alt%d:", ai);
+			for (wi = w_start; (wi == w_start) ||
+			 ((wi < sentlen) && (! sent->word[wi].unsplit_word)); wi++)
+			{
+				/* Check extra balancing */
+				assert((NULL != w.alternatives[ai]) ||
+				 (NULL == sent->word[wi].alternatives[ai]),
+				 "Extra alt %d for word %d\n", wi, ai);
+
+				if (alt_exists)
+				{
+					const char *wt = sent->word[wi].alternatives[ai];
+					const char *st = NULL;
+					char w[MAX_WORD*2];
+
+					/* Don't display information again for the same word */
+					if (findtoken && (0 == strcmp(findtoken, wt)))
+					{
+						tokenpos.wi = wi;
+						tokenpos.ai = ai;
+						return tokenpos;
+					}
+					if ((NULL != display))
+					{
+						tokenpos =
+						 print_sentence_word_alternatives(sent, FALSE, NULL, wt);
+						if ((tokenpos.wi != wi) || (tokenpos.ai != ai))
+						{
+							/* We encountered this token earlier */
+							lgdebug(5, "Skiping repeated %s\n", wt);
+							continue;
+						}
+					}
+
+					/* restore SUBSCRIPT_DOT for printing */
+					st = strrchr(wt, SUBSCRIPT_MARK);
+					if (st)
+					{
+						strcpy(w, wt);
+						w[st-wt] = SUBSCRIPT_DOT;
+						wt = w;
+					}
+					if (debugprint) lgdebug(0, " %s", wt);
+
+					/* For now each word component is called "Token".
+					 * TODO: Its type can be decoded and a more precise
+					 * term (stem, prefix, etc.) can be used.
+					 * Display the features of the token */
+					if (! findtoken && (NULL != display))
+					{
+						printf("Token \"%s\" ", wt);
+						display(sent->dict, wt);
+						printf("\n");
+					}
+					else if (word_split) printf("  %s", wt);
+				}
+			}
+			if (!alt_exists)
+				break;
+			if (word_split && NULL == display) printf("\n");
+		}
+		wi--;
+		if (debugprint) lgdebug(0, "\n");
+	}
+	if (debugprint) lgdebug(0, "\n");
+	else if (word_split) printf("\n");
+
+	return tokenpos;
+}
