@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2013 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -78,8 +78,8 @@
 
  is_conjunction[]
     An array of chars, one for each word.  TRUE if the word is a conjunction
-    ("and", "or", "nor", or "but" at the moment).  False otherwise. 
-#endif USE_FAT_LINKAGES 
+    ("and", "or", "nor", or "but" at the moment).  False otherwise.
+#endif USE_FAT_LINKAGES
 */
 
 
@@ -96,7 +96,18 @@
 /* The following define the names of the special strings in the dictionary. */
 #define LEFT_WALL_WORD   ("LEFT-WALL")
 #define RIGHT_WALL_WORD  ("RIGHT-WALL")
-#define EMPTY_WORD  ("=.zzz")
+
+/* Word subscripts come after the subscript mark (ASCII ETX)
+ * In the dictionary, a dot is used; but that dot interfers with dots
+ * in the input stream, and so we convert dictionary dots into the
+ * subscript mark, which we don't expect to see in user input.
+ */
+/* #define SUBSCRIPT_MARK '.' */
+#define SUBSCRIPT_MARK 0x3
+#define SUBSCRIPT_DOT '.'
+#define EMPTY_WORD_DOT   "=.zzz"  /* Has SUBSCRIPT_DOT in it! */
+#define EMPTY_WORD_MARK  "=zzz" /* Has SUBSCRIPT_MARK in it! */
+/* #define EMPTY_WORD_MARK  EMPTY_WORD_DOT */
 
 #ifdef USE_FAT_LINKAGES
   #define ANDABLE_CONNECTORS_WORD ("ANDABLE-CONNECTORS")
@@ -121,7 +132,6 @@
 /* MAX_WORD is large, because unicode entries can use a lot of space */
 #define MAX_WORD 180          /* maximum number of bytes in a word */
 #define MAX_LINE 2500         /* maximum number of chars in a sentence */
-#define MAX_DISJUNCT_COST 10000
 
 /* conditional compiling flags */
 #define INFIX_NOTATION
@@ -134,11 +144,11 @@
 
 #ifndef _MSC_VER
 typedef long long s64; /* signed 64-bit integer, even on 32-bit cpus */
-#define PARSE_NUM_OVERFLOW (1LL<<24)  
+#define PARSE_NUM_OVERFLOW (1LL<<24)
 #else
 /* Microsoft Visual C Version 6 doesn't support long long. */
 typedef signed __int64 s64; /* signed 64-bit integer, even on 32-bit cpus */
-#define PARSE_NUM_OVERFLOW (((s64)1)<<24)  
+#define PARSE_NUM_OVERFLOW (((s64)1)<<24)
 #endif
 
 #ifdef USE_FAT_LINKAGES
@@ -190,7 +200,7 @@ struct Disjunct_struct
     Disjunct *next;
     const char * string;
     Connector *left, *right;
-    float cost;
+    double cost;
     char marked;
 };
 
@@ -271,31 +281,29 @@ struct Andlist_struct
 };
 #endif /* USE_FAT_LINKAGES */
 
-/** 
- * This is for building the graphs of links in post-processing and 
+/**
+ * This is for building the graphs of links in post-processing and
  * fat link extraction.
  */
 struct Linkage_info_struct
 {
     int index;
+    short N_violations;
+    short unused_word_cost;
+    short link_cost;
+    double disjunct_cost;
+    double corpus_cost;
+    int island[MAX_SENTENCE];
+    size_t nwords;
 #ifdef USE_FAT_LINKAGES
     Boolean canonical;
     Boolean fat;
     Boolean improper_fat_linkage;
     Boolean inconsistent_domains;
     short and_cost;
-#endif /* USE_FAT_LINKAGES */
-    short N_violations;
-    short null_cost;
-    short unused_word_cost;
-    short link_cost;
-    float disjunct_cost;
-    double corpus_cost;
-    int island[MAX_SENTENCE];
-    size_t nwords;
-#ifdef USE_FAT_LINKAGES
     Andlist * andlist;
 #endif /* USE_FAT_LINKAGES */
+    const char *pp_violation_msg;
     char **disjunct_list_str;
 #ifdef USE_CORPUS
     Sense **sense_list;
@@ -312,7 +320,7 @@ struct List_o_links_struct
 
 #ifdef USE_FAT_LINKAGES
 /* These parameters tell power_pruning, to tell whether this is before
- * or after generating and disjuncts.  GENTLE is before RUTHLESS is 
+ * or after generating and disjuncts.  GENTLE is before RUTHLESS is
  * after.
  */
 #define GENTLE 1
