@@ -1,12 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright 2008, 2009 Linas Vepstas                                    */
+/* Copyright 2008, 2009, 2013 Linas Vepstas                              */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -52,14 +51,18 @@
 #define DEFAULTPATH DICTIONARY_DIR
 
 /* This file contains certain general utilities. */
-int   verbosity;
+int    verbosity;
+/* debug and test should not be NULL since they can be used before they
+ * are assigned a value by parse_options_get_...() */
+char * debug = (char *)"";
+char * test = (char *)"";
 
 /* ============================================================= */
 /* String utilities */
 
 char *safe_strdup(const char *u)
 {
-	if(u)
+	if (u)
 		return strdup(u);
 	return NULL;
 }
@@ -102,24 +105,15 @@ void safe_strcat(char *u, const char *v, size_t usize)
 }
 
 /**
- * prints s then prints the last |t|-|s| characters of t.
- * if s is longer than t, it truncates s.
- *
- * XXX This is not UTF8-correct, which mostly doesn't matter,
- * unless printing dictionary entries... FIXME someday.
+ * Prints s in a field width of string ti, with no truncation.
+ * FIXME: make t a number.
+ * (In a previous version of this function s got truncated to the
+ * field width.)
  */
 void left_print_string(FILE * fp, const char * s, const char * t)
 {
-	int i, j, k;
-	j = strlen(t);
-	k = strlen(s);
-	for (i=0; i<j; i++) {
-		if (i<k) {
-			fprintf(fp, "%c", s[i]);
-		} else {
-			fprintf(fp, "%c", t[i]);
-		}
-	}
+	int width = strlen(t) + strlen(s) - utf8_strlen(s);
+	fprintf(fp, "%-*s", width, s);
 }
 
 #ifdef _WIN32  /* should be !defined(HAVE_STRTOK_R) */
@@ -236,6 +230,7 @@ size_t lg_mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
 #if __APPLE__
 /* Junk, to keep the Mac OSX linker happy, because this is listed in
  * the link-grammar.def symbol export file.  */
+void lg_mbrtowc();
 void lg_mbrtowc() {}
 #endif
 
@@ -265,8 +260,8 @@ static int wctomb_check(char *s, wchar_t wc)
 
 /**
  * Downcase the first letter of the word.
- * XXX FIXME This works 'most of the time', but is not techically correct.
- * This is because towlower() and towupper() are locale dependant, and also
+ * XXX FIXME This works 'most of the time', but is not technically correct.
+ * This is because towlower() and towupper() are locale dependent, and also
  * because the byte-counts might not match up, e.g. German ß and SS.
  * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
@@ -307,8 +302,8 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
 
 /**
  * Upcase the first letter of the word.
- * XXX FIXME This works 'most of the time', but is not techically correct.
- * This is because towlower() and towupper() are locale dependant, and also
+ * XXX FIXME This works 'most of the time', but is not technically correct.
+ * This is because towlower() and towupper() are locale dependent, and also
  * because the byte-counts might not match up, e.g. German ß and SS.
  * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
@@ -559,6 +554,38 @@ void exfree(void * p, size_t size)
 }
 #endif /* TRACK_SPACE_USAGE */
 
+/* =========================================================== */
+/* Simple, cheap, easy dynamic string. */
+
+dyn_str* dyn_str_new(void)
+{
+	dyn_str *ds = malloc(sizeof(dyn_str));
+	ds->len = 250;
+	ds->end = 0;
+	ds->str = malloc(ds->len);
+	ds->str[0] = 0x0;
+	return ds;
+}
+ 
+void dyn_str_delete(dyn_str* ds)
+{
+	free(ds->str);
+	free(ds);
+}
+
+void dyn_strcat(dyn_str* ds, const char *str)
+{
+	size_t l = strlen(str);
+	if (ds->end+l+1 >= ds->len)
+	{
+		ds->len = 2 * ds->len + l;
+		ds->str = realloc(ds->str, ds->len);
+	}
+	strcpy (ds->str+ds->end, str);
+	ds->end += l;
+}
+ 
+
 /* =========================================================== */
 /* File path and dictionary open routines below */
 
@@ -678,7 +705,7 @@ path_get_dirname (const char *file_name)
 }
 #endif /* _WIN32 */
 
-/* global - but thats OK, since this is set only during initialization,
+/* global - but that's OK, since this is set only during initialization,
  * and is is thenceforth a read-only item. So it doesn't need to be
  * locked.
  */
@@ -841,7 +868,7 @@ void * object_open(const char *filename,
 			 * For unix, this should look like:
 			 * /usr/share/link-grammar:.:data:..:../data:
 			 * For windows:
-			 * C:\SOMWHERE;.;data;..;..\data;
+			 * C:\SOMEWHERE;.;data;..;..\data;
 			 */
 			snprintf(fulldictpath, MAX_PATH_NAME,
 			         "%s%c%s%c%s%c%s%c%s%c%s%c%s%c",
@@ -896,7 +923,7 @@ FILE *dictopen(const char *filename, const char *how)
 
 	/* If not the first time through, look for the other dictionaries
 	 * in the *same* directory in which the first one was found.
-	 * (The first one is typcailly "en/4.0.dict")
+	 * (The first one is typically "en/4.0.dict")
 	 * The global "path_found" records where the first dict was found.
 	 * The goal here is to avoid insanity due to user's fractured installs.
 	 */
@@ -923,6 +950,34 @@ FILE *dictopen(const char *filename, const char *how)
 
 /* ======================================================== */
 
+/**
+ * Check to see if a file exists.
+ */
+Boolean file_exists(const char * dict_name)
+{
+	Boolean retval = FALSE;
+	int fd;
+	struct stat buf;
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	/* binary, otherwise fstat file length is confused by crlf
+	 * counted as one byte. */
+	FILE *fp = dictopen(dict_name, "rb");
+#else
+	FILE *fp = dictopen(dict_name, "r");
+#endif
+	if (fp == NULL)
+		return FALSE;
+
+	/* Get the file size, in bytes. */
+	fd = fileno(fp);
+	fstat(fd, &buf);
+	if (0 < buf.st_size) retval = TRUE;
+
+  	fclose(fp);
+	return retval;
+}
+
 /**
  * Read in the whole stinkin file. This routine returns
  * malloced memory, which should be freed as soon as possible.
@@ -1107,4 +1162,15 @@ char * get_default_locale(void)
 	return locale;
 }
 
+/* ============================================================= */
+/* Alternatives utilities */
+
+size_t altlen(const char **arr)
+{
+	size_t len = 0;
+	if (arr)
+		while (arr[len] != NULL) len++;
+	return len;
+}
+
 /* ========================== END OF FILE =================== */
