@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2009-2013 Linas Vepstas                                 */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */ 
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */ 
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -144,21 +144,34 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
    (Or in linux when I compile with -ansi) */
 #endif
 
+#ifndef Boolean
+#define Boolean bool
+#endif
+
 #ifndef FALSE
-#define FALSE 0
+#define FALSE false
 #endif
 
 #ifndef TRUE
-#define TRUE 1
+#define TRUE true
 #endif
 
-#define assert(ex,string) {                                       \
-    if (!(ex)) {                                                  \
-        prt_error("Assertion failed: %s\n", string);              \
-        exit(1);                                                  \
-    }                                                             \
+#define STR(x) #x
+#define STRINGIFY(x) STR(x)
+
+#define FILELINE __FILE__ ":" STRINGIFY(__LINE__)
+#define assert(ex, ...) {                                                   \
+	if (!(ex)) {                                                             \
+		prt_error("\nAssertion ("#ex") failed at "FILELINE": "  __VA_ARGS__); \
+		fflush(stdout);                                                       \
+		exit(1);                                                              \
+	}                                                                        \
 }
 
+#if defined(__UCLIBC__)
+#define fmaxf(a,b) ((a) > (b) ? (a) : (b))
+#endif
+
 #if !defined(MIN)
 #define MIN(X,Y)  ( ((X) < (Y)) ? (X) : (Y))
 #endif
@@ -309,6 +322,20 @@ void safe_strcpy(char *u, const char * v, size_t usize);
 void safe_strcat(char *u, const char *v, size_t usize);
 char *safe_strdup(const char *u);
 
+/* Simple, cheap, easy dynamic string. */
+typedef struct
+{
+  char *str;
+  size_t end;
+  size_t len;
+} dyn_str;
+
+dyn_str* dyn_str_new(void);
+void dyn_str_delete(dyn_str*);
+void dyn_strcat(dyn_str*, const char*);
+
+size_t altlen(const char **arr);
+
 /* routines for allocating basic objects */
 void init_memusage(void);
 void * xalloc(size_t) GNUC_MALLOC;
@@ -323,7 +350,7 @@ void * xrealloc(void *, size_t oldsize, size_t newsize) GNUC_MALLOC;
   /* **MUST** define for win32, Mac OSX, because xfree is listed in
    * link-grammar.def and the win/osx linker fails if there is no
    * xfree. So, keep the linker happy. (xfree is used in both dict-file
-   * and sat solver)
+   * and sat solver.)
    */
   #define TRACK_SPACE_USAGE 1
 #endif /* defined(_WIN32) || __APPLE__ */
@@ -348,6 +375,7 @@ void * object_open(const char *filename,
                    void * (*opencb)(const char *, void *),
                    void * user_data);
 
+Boolean file_exists(const char * dict_name);
 char * get_file_contents(const char *filename);
 
 /**
