@@ -1,11 +1,11 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2009, 2013 Linas Vepstas                                */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */ 
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */ 
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -15,6 +15,8 @@
 #define _LINK_GRAMMAR_WORD_UTILS_H_
 
 #include "api-types.h"
+#include "dict-structures.h"
+#include "structures.h"
 
 /* Exp utilities ... */
 void free_Exp(Exp *);
@@ -51,7 +53,7 @@ Boolean word_has_connector(Dict_node *, const char *, char);
 
 
 /* Dictionary utilities ... */
-int word_contains(Dictionary dict, const char * word, const char * macro);
+Boolean word_contains(Dictionary dict, const char * word, const char * macro);
 
 static inline int string_hash(const char *s)
 {
