@@ -5,8 +5,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software,    */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html    */
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -16,12 +15,14 @@
 #include <wctype.h>
 
 #include "command-line.h"
-#include "link-includes.h"
-#include "utilities.h"  /* For MSWindows portability */
+#include <link-grammar/link-includes.h>
+#include "../link-grammar/utilities.h"  /* For MSWindows portability */
 
 static struct
 {
 	int verbosity;
+	char * debug;
+	char * test;
 	int timeout;
 	int memory;
 	int linkage_limit;
@@ -41,7 +42,7 @@ static struct
 	int use_viterbi;
 	int echo_on;
 	Cost_Model_type cost_model;
-	float max_cost;
+	double max_cost;
 	int screen_width;
 	int display_on;
 	ConstituentDisplayStyle display_constituents;
@@ -52,7 +53,7 @@ static struct
 	int display_walls;
 	int display_disjuncts;
 	int display_senses;
-	int display_suffixes;
+	int display_morphology;
 } local;
 
 typedef enum
@@ -60,6 +61,7 @@ typedef enum
 	Int,
 	Bool,
 	Float,
+	String,
 } ParamType;
 
 typedef struct
@@ -86,6 +88,7 @@ static Switch default_switches[] =
    {"links",      Bool, "Display of complete link data",   &local.display_links},
    {"max-length", Int,  "Maximum sentence length",         &local.max_sentence_length},
    {"memory",     Int,  "Max memory allowed",              &local.memory},
+   {"morphology", Bool, "Display word morphology",         &local.display_morphology},
    {"null",       Bool, "Allow null links",                &local.allow_null},
    {"null-block", Int,  "Size of blocks with null count 1", &local.null_block},
    {"panic",      Bool, "Use of \"panic mode\"",           &local.panic_mode},
@@ -95,7 +98,6 @@ static Switch default_switches[] =
 #if defined HAVE_HUNSPELL || defined HAVE_ASPELL
    {"spell",      Bool, "Use spell-guesser for unknown words",  &local.spell_guess},
 #endif /* HAVE_HUNSPELL */
-   {"suffixes",   Bool, "Display of word stems+suffixes",  &local.display_suffixes},
    {"timeout",    Int,  "Abort parsing after this many seconds", &local.timeout},
 #ifdef USE_FAT_LINKAGES
    {"union",      Bool, "Display of 'union' linkage",      &local.display_union},
@@ -105,6 +107,8 @@ static Switch default_switches[] =
    {"use-sat",    Bool, "Use Boolean SAT-based parser",    &local.use_sat_solver},
 #endif /* USE_SAT_SOLVER */
    {"verbosity",  Int,  "Level of detail in output",       &local.verbosity},
+   {"debug",      String, "comma-separated function list to debug", &local.debug},
+   {"test",       String, "comma-separated features to test", &local.test},
 #ifdef USE_VITERBI
    {"viterbi",    Bool, "Use Viterbit-based parser",       &local.use_viterbi},
 #endif
@@ -246,23 +250,29 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 
 	if (strcmp(s, "variables") == 0)
 	{
-        printf(" Variable     Controls                                      Value\n");
-        printf(" --------     --------                                      -----\n");
+		printf(" Variable     Controls                                      Value\n");
+		printf(" --------     --------                                      -----\n");
 		for (i = 0; as[i].string != NULL; i++)
 		{
 			printf(" ");
-            left_print_string(stdout, as[i].string, "             ");
-            left_print_string(stdout, as[i].description,
-                   "                                              ");
+			left_print_string(stdout, as[i].string, "             ");
+			left_print_string(stdout, as[i].description,
+			            "                                              ");
 			if (Float == as[i].param_type)
 			{
 				/* Float point print! */
-				printf("%5.2f", (double) (*((float *)as[i].ptr)));
+				printf("%5.2f", *((double *)as[i].ptr));
 			}
 			else
+			if ((Bool == as[i].param_type) || Int == as[i].param_type)
 			{
 				printf("%5d", ival(as[i]));
 			}
+			else
+			if (String == as[i].param_type)
+			{
+				printf("%s", *(char **)as[i].ptr);
+			}
 			if (Bool == as[i].param_type)
 			{
 				if (ival(as[i])) printf(" (On)"); else printf(" (Off)");
@@ -281,12 +291,12 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 		printf("can be abbreviated.  Here is a list of the commands:\n\n");
 		for (i=0; user_command[i].s != NULL; i++) {
 			printf(" !");
-            left_print_string(stdout, user_command[i].s, "               ");
-            left_print_string(stdout, user_command[i].str, "                                                    ");
+			left_print_string(stdout, user_command[i].s, "               ");
+			left_print_string(stdout, user_command[i].str, "                                                    ");
 			printf("\n");
 		}
-		printf(" !!<string>      Print all the dictionary words matching <string>.\n");
-		printf("                 Also print the number of disjuncts of each.\n");
+		printf(" !!<string>      Print all the dictionary word that matchs <string>.\n");
+		printf("                 A wildcard * may be used to find multiple matches.\n");
 		printf("\n");
 		printf(" !<var>          Toggle the specified boolean variable.\n");
 		printf(" !<var>=<val>    Assign that value to that variable.\n");
@@ -295,8 +305,8 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 
 	if (s[0] == '!')
 	{
-		dict_display_word_info(dict, s+1);
-		dict_display_word_expr(dict, s+1);
+		dict_display_word_info(dict, s+1, opts);
+		dict_display_word_expr(dict, s+1, opts);
 		return 0;
 	}
 
@@ -333,7 +343,7 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 			return -1;
 		}
 
-		if (as[j].param_type != Float)
+		if ((as[j].param_type == Int) || (as[j].param_type == Bool))
 		{
 			int val = -1;
 			if (is_numerical_rhs(y)) val = atoi(y);
@@ -352,19 +362,32 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 			return 0;
 		}
 		else
+		if (as[j].param_type == Float)
 		{
-			float val = -1.0;
-			val = (float) atof(y);
-			if (val < 0.0f)
+			double val = -1.0;
+			val = (double) atof(y);
+			if (val < 0.0)
 			{
 				printf("Invalid value %s for variable %s Type \"!help\" or \"!variables\"\n", y, as[j].string);
 				return -1;
 			}
 
-			*((float *) as[j].ptr) = val;
+			*((double *) as[j].ptr) = val;
 			printf("%s set to %5.2f\n", as[j].string, val);
 			return 0;
 		}
+		else
+		if (as[j].param_type == String)
+		{
+			*((char **) as[j].ptr) = y;
+			printf("%s set to %s\n", (char *)as[j].string, y);
+			return 0;
+		}
+		else
+		{
+			printf("Internal error: Unknown variable type %d\n", as[j].param_type);
+			return -1;
+		}
 	}
 
 	/* Look for valid commands, but ones that needed an argument */
@@ -393,6 +416,8 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 static void put_opts_in_local_vars(Parse_Options opts)
 {
 	local.verbosity = parse_options_get_verbosity(opts);
+	local.debug = parse_options_get_debug(opts);
+	local.test = parse_options_get_test(opts);
 	local.timeout = parse_options_get_max_parse_time(opts);;
 	local.memory = parse_options_get_max_memory(opts);;
 	local.linkage_limit = parse_options_get_linkage_limit(opts);
@@ -401,7 +426,7 @@ static void put_opts_in_local_vars(Parse_Options opts)
 	local.spell_guess = parse_options_get_spell_guess(opts);
 	local.short_length = parse_options_get_short_length(opts);
 	local.cost_model = parse_options_get_cost_model_type(opts);
-	local.max_cost = parse_options_get_disjunct_costf(opts);
+	local.max_cost = parse_options_get_disjunct_cost(opts);
 	local.echo_on = parse_options_get_echo_on(opts);
 	local.batch_mode = parse_options_get_batch_mode(opts);
 	local.panic_mode = parse_options_get_panic_mode(opts);
@@ -422,14 +447,16 @@ static void put_opts_in_local_vars(Parse_Options opts)
 	local.display_bad = parse_options_get_display_bad(opts);
 	local.display_disjuncts = parse_options_get_display_disjuncts(opts);
 	local.display_links = parse_options_get_display_links(opts);
+	local.display_morphology = parse_options_get_display_morphology(opts);
 	local.display_senses = parse_options_get_display_senses(opts);
-	local.display_suffixes = parse_options_get_display_suffixes(opts);
 	local.display_walls = parse_options_get_display_walls(opts);
 }
 
 static void put_local_vars_in_opts(Parse_Options opts)
 {
 	parse_options_set_verbosity(opts, local.verbosity);
+	parse_options_set_debug(opts, local.debug);
+	parse_options_set_test(opts, local.test);
 	parse_options_set_max_parse_time(opts, local.timeout);
 	parse_options_set_max_memory(opts, local.memory);
 	parse_options_set_linkage_limit(opts, local.linkage_limit);
@@ -439,7 +466,7 @@ static void put_local_vars_in_opts(Parse_Options opts)
 	parse_options_set_short_length(opts, local.short_length);
 	parse_options_set_echo_on(opts, local.echo_on);
 	parse_options_set_cost_model_type(opts, local.cost_model);
-	parse_options_set_disjunct_costf(opts, local.max_cost);
+	parse_options_set_disjunct_cost(opts, local.max_cost);
 	parse_options_set_batch_mode(opts, local.batch_mode);
 	parse_options_set_panic_mode(opts, local.panic_mode);
 	parse_options_set_screen_width(opts, local.screen_width);
@@ -461,8 +488,8 @@ static void put_local_vars_in_opts(Parse_Options opts)
 	parse_options_set_display_bad(opts, local.display_bad);
 	parse_options_set_display_disjuncts(opts, local.display_disjuncts);
 	parse_options_set_display_links(opts, local.display_links);
+	parse_options_set_display_morphology(opts, local.display_morphology);
 	parse_options_set_display_senses(opts, local.display_senses);
-	parse_options_set_display_suffixes(opts, local.display_suffixes);
 	parse_options_set_display_walls(opts, local.display_walls);
 }
 
