@@ -5,8 +5,7 @@
 /* All rights reserved                                                     */
 /*                                                                         */
 /* Use of the link grammar parsing system is subject to the terms of the   */
-/* license set forth in the LICENSE file included with this software,      */
-/* and also available at http://www.link.cs.cmu.edu/link/license.html      */
+/* license set forth in the LICENSE file included with this software.      */
 /* This license allows free redistribution and use in source and binary    */
 /* forms, with or without modification, subject to certain conditions.     */
 /*                                                                         */
@@ -49,11 +48,11 @@
 #define LINK_GRAMMAR_DLL_EXPORT 0
 #endif
 
-#include "link-includes.h"
+#include <link-grammar/link-includes.h>
 #include "command-line.h"
-#include "expand.h"
 #include "lg_readline.h"
-#include "utilities.h"     /* For MSVC portability */
+#include "../link-grammar/expand.h"
+#include "../link-grammar/utilities.h"     /* For MSVC portability */
 #include "../viterbi/viterbi.h"
 
 #define MAX_INPUT 1024
@@ -84,7 +83,7 @@ static char* oem_to_utf8(char *instring)
 	int cv;
 	unsigned int consolecp;
 
-	consolecp = GetConsoleOutputCP(); 
+	consolecp = GetConsoleOutputCP();
 
 	/* Convert input string to wide chars. */
 	len = strlen(instring) + 1;
@@ -144,10 +143,10 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 #else
 	static char input_string[MAX_INPUT];
 
-	if (NULL == in) return NULL; 
+	if (NULL == in) return NULL;
 
-	if ((!parse_options_get_batch_mode(opts)) && 
-	    (verbosity > 0) && 
+	if ((!parse_options_get_batch_mode(opts)) &&
+	    (verbosity > 0) &&
 	    (!input_pending))
 	{
 		fprintf(out, "linkparser> ");
@@ -182,10 +181,10 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 		 * wide chars here, and convert to multi-byte UTF-8 on the fly.
 		 */
 		wchar_t winput_string[MAX_INPUT];
-		if (fgetws(winput_string, MAX_INPUT, in)) 
+		if (fgetws(winput_string, MAX_INPUT, in))
 		{
 			size_t nc = wcstombs(input_string, winput_string, MAX_INPUT);
-			if (nc && (((size_t) -1) != nc)) 
+			if (nc && (((size_t) -1) != nc))
 			{
 				char *cr, *lf;
 				cr = strchr(input_string, '\r');
@@ -272,6 +271,12 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 	{
 		linkage_set_current_sublinkage(linkage, j);
 #endif /* USE_FAT_LINKAGES */
+		if (parse_options_get_display_bad(opts))
+		{
+			string = linkage_print_pp_msgs(linkage);
+			fprintf(stdout, "%s\n", string);
+			linkage_free_pp_msgs(string);
+		}
 		if (parse_options_get_display_on(opts))
 		{
 			string = linkage_print_diagram(linkage);
@@ -421,7 +426,7 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 #ifdef USE_FAT_LINKAGES
 			if (corpus_cost < 0.0f)
 			{
-				fprintf(stdout, "cost vector = (UNUSED=%d DIS=%d FAT=%d AND=%d LEN=%d)\n",
+				fprintf(stdout, "cost vector = (UNUSED=%d DIS=%5.2f FAT=%d AND=%d LEN=%d)\n",
 				       linkage_unused_word_cost(linkage),
 				       linkage_disjunct_cost(linkage),
 				       linkage_is_fat(linkage),
@@ -430,7 +435,7 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 			}
 			else
 			{
-				fprintf(stdout, "cost vector = (CORP=%6.4f UNUSED=%d DIS=%d FAT=%d AND=%d LEN=%d)\n",
+				fprintf(stdout, "cost vector = (CORP=%6.4f UNUSED=%d DIS=%5.2f FAT=%d AND=%d LEN=%d)\n",
 				       corpus_cost,
 				       linkage_unused_word_cost(linkage),
 				       linkage_disjunct_cost(linkage),
@@ -441,14 +446,14 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 #else
 			if (corpus_cost < 0.0f)
 			{
-				fprintf(stdout, "cost vector = (UNUSED=%d DIS=%d LEN=%d)\n",
+				fprintf(stdout, "cost vector = (UNUSED=%d DIS=%5.2f LEN=%d)\n",
 				       linkage_unused_word_cost(linkage),
 				       linkage_disjunct_cost(linkage),
 				       linkage_link_cost(linkage));
 			}
 			else
 			{
-				fprintf(stdout, "cost vector = (CORP=%6.4f UNUSED=%d DIS=%d LEN=%d)\n",
+				fprintf(stdout, "cost vector = (CORP=%6.4f UNUSED=%d DIS=%5.2f LEN=%d)\n",
 				       corpus_cost,
 				       linkage_unused_word_cost(linkage),
 				       linkage_disjunct_cost(linkage),
@@ -484,7 +489,7 @@ static int there_was_an_error(Label label, Sentence sent, Parse_Options opts)
 			batch_errors++;
 			return UNGRAMMATICAL;
 		}
-		if ((sentence_disjunct_cost(sent, 0) == 0) &&
+		if ((sentence_disjunct_cost(sent, 0) == 0.0) &&
 			(label == PARSE_WITH_DISJUNCT_COST_GT_0)) {
 			batch_errors++;
 			return PARSE_WITH_DISJUNCT_COST_GT_0;
@@ -498,7 +503,7 @@ static int there_was_an_error(Label label, Sentence sent, Parse_Options opts)
 	return FALSE;
 }
 
-static void batch_process_some_linkages(Label label, 
+static void batch_process_some_linkages(Label label,
                                         Sentence sent,
                                         Parse_Options opts)
 {
@@ -565,7 +570,7 @@ static Label strip_off_label(char * input_string)
 
 static void setup_panic_parse_options(Parse_Options opts)
 {
-	parse_options_set_disjunct_costf(opts, 3.0f);
+	parse_options_set_disjunct_cost(opts, 3.0f);
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, 100);
 	parse_options_set_max_parse_time(opts, 60);
@@ -579,11 +584,12 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_spell_guess(opts, FALSE);
 }
 
-static void print_usage(char *str) {
+static void print_usage(char *str)
+{
 	fprintf(stderr,
 			"Usage: %s [language|dictionary location]\n"
-			"		  [-<special \"!\" command>]\n"
-			"		  [--version]\n", str);
+			"                   [-<special \"!\" command>]\n"
+			"                   [--version]\n", str);
 
 	fprintf(stderr, "\nSpecial commands are:\n");
 	opts = parse_options_create();
@@ -592,8 +598,8 @@ static void print_usage(char *str) {
 }
 
 /**
- * On Unix, this checks for the current window size, 
- * and sets the output screen width accordingly. 
+ * On Unix, this checks for the current window size,
+ * and sets the output screen width accordingly.
  * Not sure how MS Windows does this.
  */
 static void check_winsize(Parse_Options popts)
@@ -654,25 +660,25 @@ int main(int argc, char * argv[])
 	}
 
 #if !defined(_MSC_VER) && !defined(__MINGW32__)
-	/* Get the locale from the environment... 
+	/* Get the locale from the environment...
 	 * Perhaps we should someday get it from the dictionary ??
 	 */
 	locale = setlocale(LC_ALL, "");
 
 	/* Check to make sure the current locale is UTF8; if its not,
-	 * then force-set this to the english utf8 locale 
+	 * then force-set this to the english utf8 locale
 	 */
 	codeset = nl_langinfo(CODESET);
 	if (!strstr(codeset, "UTF") && !strstr(codeset, "utf"))
 	{
-		fprintf(stderr, 
+		fprintf(stderr,
 		    "%s: Warning: locale %s was not UTF-8; force-setting to en_US.UTF-8\n",
 		     argv[0], codeset);
 		locale = setlocale(LC_CTYPE, "en_US.UTF-8");
 	}
 #else
  #pragma message("WARNING: Windows console (cmd.exe) does not support unicode input!\nWill attempt to convert from the native encoding!");
-	fprintf(stderr, 
+	fprintf(stderr,
 	    "%s: Warning: Windows console (cmd.exe) does not support unicode\n"
 	    "input!  Will attempt to convert from the native encoding!", argv[0]);
 #endif
@@ -719,7 +725,7 @@ int main(int argc, char * argv[])
 	parse_options_set_max_parse_time(opts, 30);
 	parse_options_set_linkage_limit(opts, 1000);
 	parse_options_set_short_length(opts, 10);
-	parse_options_set_disjunct_costf(opts, 2.0f);
+	parse_options_set_disjunct_cost(opts, 2.0);
 	parse_options_set_min_null_count(opts, 0);
 	parse_options_set_max_null_count(opts, 0);
 
@@ -834,7 +840,7 @@ int main(int argc, char * argv[])
 			sent = sentence_create(input_string, dict);
 
 			/* First parse with cost 0 or 1 and no null links */
-			// parse_options_set_disjunct_costf(opts, 2.0f);
+			// parse_options_set_disjunct_cost(opts, 2.0);
 			parse_options_set_min_null_count(opts, 0);
 			parse_options_set_max_null_count(opts, 0);
 			parse_options_reset_resources(opts);
@@ -850,20 +856,20 @@ int main(int argc, char * argv[])
 				continue;
 			}
 #if 0
-			/* Try again, this time ommitting the requirement for
+			/* Try again, this time omitting the requirement for
 			 * definite articles, etc. This should allow for the parsing
 			 * of newspaper headlines and other clipped speech.
 			 *
 			 * XXX Unfortunately, this also allows for the parsing of
-			 * all sorts of ungrammatical sentences which should not 
+			 * all sorts of ungrammatical sentences which should not
 			 * parse, and leads to bad parses of many other unparsable
 			 * but otherwise grammatical sentences.  Thus, this trick
-			 * pretty much fails; we leave it here to document the 
+			 * pretty much fails; we leave it here to document the
 			 * experiment.
 			 */
 			if (num_linkages == 0)
 			{
-				parse_options_set_disjunct_costf(opts, 3.5f);
+				parse_options_set_disjunct_cost(opts, 3.5);
 				num_linkages = sentence_parse(sent, opts);
 				if (num_linkages < 0) continue;
 			}
@@ -878,7 +884,7 @@ int main(int argc, char * argv[])
 			{
 				int expanded;
 				if (verbosity > 0) fprintf(stdout, "No standard linkages, expanding disjunct set.\n");
-				parse_options_set_disjunct_costf(opts, 2.9f);
+				parse_options_set_disjunct_cost(opts, 2.9);
 				expanded = lg_expand_disjunct_list(sent);
 				if (expanded)
 				{
